@@ -17,23 +17,28 @@
 import os
 import logging
 from lithops.version import __version__
-from lithops.config import cloud_logging_config
+from lithops.utils import setup_lithops_logger
 from lithops.worker import function_handler
 from lithops.worker import function_invoker
+from lithops.worker.utils import get_runtime_preinstalls
 
-cloud_logging_config(logging.INFO)
-logger = logging.getLogger('__main__')
+logger = logging.getLogger('lithops.worker')
 
 
 def lambda_handler(event, context):
-    os.environ['__PW_ACTIVATION_ID'] = context.aws_request_id
-    os.environ['__OW_ACTIVATION_ID'] = context.aws_request_id
+    os.environ['__LITHOPS_ACTIVATION_ID'] = context.aws_request_id
+    os.environ['__LITHOPS_BACKEND'] = 'AWS Lambda'
 
-    if 'remote_invoker' in event:
-        logger.info("Lithops v{} - Starting invoker".format(__version__))
+    setup_lithops_logger(event.get('log_level', logging.INFO))
+
+    if 'get_preinstalls' in event:
+        logger.info("Lithops v{} - Generating metadata".format(__version__))
+        return get_runtime_preinstalls()
+    elif 'remote_invoker' in event:
+        logger.info("Lithops v{} - Starting AWS Lambda invoker".format(__version__))
         function_invoker(event)
     else:
-        logger.info("Lithops v{} - Starting execution".format(__version__))
+        logger.info("Lithops v{} - Starting AWS Lambda execution".format(__version__))
         function_handler(event)
 
     return {"Execution": "Finished"}
