@@ -15,23 +15,32 @@
 #
 
 import os
+import sys
 import logging
 from lithops.version import __version__
-from lithops.config import cloud_logging_config
+from lithops.utils import setup_lithops_logger
 from lithops.worker import function_handler
 from lithops.worker import function_invoker
+from lithops.worker.utils import get_runtime_preinstalls
+from lithops.constants import LOGGER_FORMAT_SHORT
 
-cloud_logging_config(logging.INFO)
-logger = logging.getLogger('__main__')
+logger = logging.getLogger('lithops.worker')
 
 
 def main(args):
     os.environ['__LITHOPS_ACTIVATION_ID'] = os.environ['__OW_ACTIVATION_ID']
-    if 'remote_invoker' in args:
-        logger.info("Lithops v{} - Starting invoker".format(__version__))
+    os.environ['__LITHOPS_BACKEND'] = 'IBM CF'
+
+    setup_lithops_logger(args['log_level'], LOGGER_FORMAT_SHORT, sys.stdout)
+
+    if 'get_preinstalls' in args:
+        logger.info("Lithops v{} - Generating metadata".format(__version__))
+        return get_runtime_preinstalls()
+    elif 'remote_invoker' in args:
+        logger.info("Lithops v{} - Starting IBM CF invoker".format(__version__))
         function_invoker(args)
     else:
-        logger.info("Lithops v{} - Starting execution".format(__version__))
+        logger.info("Lithops v{} - Starting IBM CF execution".format(__version__))
         function_handler(args)
 
     return {"Execution": "Finished"}
