@@ -20,6 +20,7 @@ import sys
 import ssl
 import json
 import time
+import base64
 import yaml
 import urllib3
 import logging
@@ -27,15 +28,18 @@ import requests
 import http.client
 from urllib.parse import urlparse
 from kubernetes import client, config, watch
+from kubernetes.client.rest import ApiException
+
 from lithops.utils import version_str
 from lithops.version import __version__
-from lithops.config import CACHE_DIR, load_yaml_config, dump_yaml_config
-from lithops.serverless.utils import create_function_handler_zip
+from lithops.config import load_yaml_config, dump_yaml_config
+from lithops.constants import CACHE_DIR
+from lithops.utils import create_handler_zip
+from lithops.constants import COMPUTE_CLI_MSG
+
 from . import config as kconfig
 
 urllib3.disable_warnings()
-logging.getLogger('kubernetes').setLevel(logging.CRITICAL)
-logging.getLogger('urllib3.connectionpool').setLevel(logging.CRITICAL)
 
 logger = logging.getLogger(__name__)
 
@@ -45,17 +49,19 @@ class KnativeServingBackend:
     A wrap-up around Knative Serving APIs.
     """
 
-    def __init__(self, knative_config, storage_config):
-        self.log_active = logger.getEffectiveLevel() != logging.WARNING
+    def __init__(self, knative_config, internal_storage):
         self.name = 'knative'
+        self.type = 'faas'
         self.knative_config = knative_config
         self.istio_endpoint = self.knative_config.get('istio_endpoint')
+        self.kubecfg_path = self.knative_config.get('kubecfg_path')
 
         # k8s config can be incluster, in ~/.kube/config or generate kube-config.yaml file and
         # set env variable KUBECONFIG=<path-to-kube-confg>
         try:
-            config.load_kube_config()
-            current_context = config.list_kube_config_contexts()[1].get('context')
+            config.load_kube_config(config_file=self.kubecfg_path)
+            contexts = config.list_kube_config_contexts(config_file=self.kubecfg_path)
+            current_context = contexts[1].get('context')
             self.namespace = current_context.get('namespace', 'default')
             self.cluster = current_context.get('cluster')
             self.knative_config['namespace'] = self.namespace
@@ -67,12 +73,15 @@ class KnativeServingBackend:
             self.cluster = self.knative_config.get('cluster', 'default')
             self.is_incluster = True
 
-        self.api = client.CustomObjectsApi()
-        self.v1 = client.CoreV1Api()
+        logger.debug("Set namespace to {}".format(self.namespace))
+        logger.debug("Set cluster to {}".format(self.cluster))
+
+        self.custom_api = client.CustomObjectsApi()
+        self.core_api = client.CoreV1Api()
 
         if self.istio_endpoint is None:
             try:
-                ingress = self.v1.read_namespaced_service('istio-ingressgateway', 'istio-system')
+                ingress = self.core_api.read_namespaced_service('istio-ingressgateway', 'istio-system')
                 http_port = list(filter(lambda port: port.port == 80, ingress.spec.ports))[0].node_port
                 # https_port = list(filter(lambda port: port.port == 443, ingress.spec.ports))[0].node_port
 
@@ -81,13 +90,14 @@ class KnativeServingBackend:
                     ip = ingress.status.load_balancer.ingress[0].ip
                 else:
                     # for minikube or a baremetal cluster that has no external load balancer
-                    node = self.v1.list_node()
+                    node = self.core_api.list_node()
                     ip = node.items[0].status.addresses[0].address
 
-                self.istio_endpoint = 'http://{}:{}'.format(ip, http_port)
-                self.knative_config['istio_endpoint'] = self.istio_endpoint
+                if ip and http_port:
+                    self.istio_endpoint = 'http://{}:{}'.format(ip, http_port)
+                    self.knative_config['istio_endpoint'] = self.istio_endpoint
             except Exception:
-                logger.info("istio-ingressgateway endpoint not found")
+                pass
 
         if 'service_host_suffix' not in self.knative_config:
             self.serice_host_filename = os.path.join(CACHE_DIR, 'knative', self.cluster, 'service_host')
@@ -101,13 +111,12 @@ class KnativeServingBackend:
 
         logger.debug('Loaded service host suffix: {}'.format(self.service_host_suffix))
 
+        msg = COMPUTE_CLI_MSG.format('Knative')
         if self.istio_endpoint:
-            log_msg = 'Lithops v{} init for Knative - Istio Endpoint: {}'.format(__version__, self.istio_endpoint)
-        else:
-            log_msg = 'Lithops v{} init for Knative'.format(__version__)
-        if not self.log_active:
-            print(log_msg)
-        logger.info(log_msg)
+            msg += ' - Istio Endpoint: {}'.format(self.istio_endpoint)
+        elif self.cluster:
+            msg += ' - Cluster: {}'.format(self.cluster)
+        logger.info("{}".format(msg))
 
     def _format_service_name(self, runtime_name, runtime_memory):
         runtime_name = runtime_name.replace('/', '--').replace(':', '--')
@@ -120,10 +129,10 @@ class KnativeServingBackend:
         return image_name, int(memory.replace('mb', ''))
 
     def _get_default_runtime_image_name(self):
-        docker_user = self.knative_config['docker_user']
+        docker_user = self.knative_config.get('docker_user')
         python_version = version_str(sys.version_info).replace('.', '')
-        revision = 'latest' if 'SNAPSHOT' in __version__ else __version__.replace('.', '')
-        return '{}/{}-v{}:{}'.format(docker_user, kconfig.RUNTIME_NAME_DEFAULT, python_version, revision)
+        revision = 'latest' if 'dev' in __version__ else __version__.replace('.', '')
+        return '{}/{}-v{}:{}'.format(docker_user, kconfig.RUNTIME_NAME, python_version, revision)
 
     def _get_service_host(self, service_name):
         """
@@ -131,10 +140,9 @@ class KnativeServingBackend:
         """
         logger.debug('Getting service host for: {}'.format(service_name))
         try:
-            #t0 = time.time()
-            svc = self.api.get_namespaced_custom_object(
-                        group="serving.knative.dev",
-                        version="v1alpha1",
+            svc = self.custom_api.get_namespaced_custom_object(
+                        group=kconfig.DEFAULT_GROUP,
+                        version=kconfig.DEFAULT_VERSION,
                         name=service_name,
                         namespace=self.namespace,
                         plural="services"
@@ -143,7 +151,6 @@ class KnativeServingBackend:
                 service_host = svc['status']['url'][7:]
             else:
                 raise Exception('Unable to get service details from {}'.format(service_name))
-            #print(time.time()-t0)
         except Exception as e:
             if json.loads(e.body)['code'] == 404:
                 log_msg = 'Knative service: resource "{}" Not Found'.format(service_name)
@@ -160,26 +167,30 @@ class KnativeServingBackend:
         """
         logger.debug("Creating Tekton account resources: Secret and ServiceAccount")
         string_data = {'username': self.knative_config['docker_user'],
-                       'password': self.knative_config['docker_token']}
+                       'password': self.knative_config['docker_password']}
         secret_res = yaml.safe_load(kconfig.secret_res)
         secret_res['stringData'] = string_data
 
-        if self.knative_config['docker_repo'] != kconfig.DOCKER_REPO_DEFAULT:
-            secret_res['metadata']['annotations']['tekton.dev/docker-0'] = self.knative_config['docker_repo']
+        secret_res['metadata']['annotations']['tekton.dev/docker-0'] = "docker.io"
 
         account_res = yaml.safe_load(kconfig.account_res)
         secret_res_name = secret_res['metadata']['name']
         account_res_name = account_res['metadata']['name']
 
         try:
-            self.v1.delete_namespaced_secret(secret_res_name, self.namespace)
-            self.v1.delete_namespaced_service_account(account_res_name, self.namespace)
+            self.core_api.delete_namespaced_secret(secret_res_name, self.namespace)
         except Exception:
             # account resource Not Found - Not deleted
             pass
 
-        self.v1.create_namespaced_secret(self.namespace, secret_res)
-        self.v1.create_namespaced_service_account(self.namespace, account_res)
+        try:
+            self.core_api.delete_namespaced_service_account(account_res_name, self.namespace)
+        except Exception:
+            # account resource Not Found - Not deleted
+            pass
+
+        self.core_api.create_namespaced_secret(self.namespace, secret_res)
+        self.core_api.create_namespaced_service_account(self.namespace, account_res)
 
     def _create_build_resources(self):
         logger.debug("Creating Tekton build resources: PipelineResource and Task")
@@ -198,7 +209,7 @@ class KnativeServingBackend:
         logger.debug('Setting git rev to: {}'.format(self.knative_config['git_rev']))
 
         try:
-            self.api.delete_namespaced_custom_object(
+            self.custom_api.delete_namespaced_custom_object(
                     group="tekton.dev",
                     version="v1alpha1",
                     name=task_name,
@@ -211,7 +222,7 @@ class KnativeServingBackend:
             pass
 
         try:
-            self.api.delete_namespaced_custom_object(
+            self.custom_api.delete_namespaced_custom_object(
                     group="tekton.dev",
                     version="v1alpha1",
                     name=git_res_name,
@@ -223,7 +234,7 @@ class KnativeServingBackend:
             # ksvc resource Not Found - Not deleted
             pass
 
-        self.api.create_namespaced_custom_object(
+        self.custom_api.create_namespaced_custom_object(
                 group="tekton.dev",
                 version="v1alpha1",
                 namespace=self.namespace,
@@ -231,7 +242,7 @@ class KnativeServingBackend:
                 body=git_res
             )
 
-        self.api.create_namespaced_custom_object(
+        self.custom_api.create_namespaced_custom_object(
                 group="tekton.dev",
                 version="v1alpha1",
                 namespace=self.namespace,
@@ -243,9 +254,16 @@ class KnativeServingBackend:
         """
         Builds the default runtime and pushes it to the docker container registry
         """
-        image_name, revision = docker_image_name.split(':')
+        if docker_image_name.count('/') > 1:
+            # container registry is in the provided runtime name
+            cr, rn = docker_image_name.split('/', 1)
+        else:
+            cr = 'docker.io'
+            rn = docker_image_name
 
-        if self.knative_config['docker_repo'] == 'docker.io' and revision != 'latest':
+        image_name, revision = rn.split(':')
+
+        if cr == 'docker.io' and revision != 'latest':
             resp = requests.get('https://index.docker.io/v1/repositories/{}/tags/{}'
                                 .format(docker_image_name, revision))
             if resp.status_code == 200:
@@ -255,8 +273,8 @@ class KnativeServingBackend:
 
         logger.debug("Building default Lithops runtime from git with Tekton")
 
-        if not {"docker_user", "docker_token"} <= set(self.knative_config):
-            raise Exception("You must provide 'docker_user' and 'docker_token'"
+        if not all(key in self.knative_config for key in ["docker_user", "docker_password"]):
+            raise Exception("You must provide 'docker_user' and 'docker_password'"
                             " to build the default runtime")
 
         task_run = yaml.safe_load(kconfig.task_run)
@@ -266,7 +284,7 @@ class KnativeServingBackend:
                               'value': 'lithops/compute/backends/knative/tekton/Dockerfile.python{}'.format(python_version)}
         task_run['spec']['inputs']['params'].append(path_to_dockerfile)
         image_url = {'name': 'imageUrl',
-                     'value': '/'.join([self.knative_config['docker_repo'], image_name])}
+                     'value': '/'.join([cr, image_name])}
         task_run['spec']['inputs']['params'].append(image_url)
         image_tag = {'name': 'imageTag',
                      'value':  revision}
@@ -277,7 +295,7 @@ class KnativeServingBackend:
 
         task_run_name = task_run['metadata']['name']
         try:
-            self.api.delete_namespaced_custom_object(
+            self.custom_api.delete_namespaced_custom_object(
                     group="tekton.dev",
                     version="v1alpha1",
                     name=task_run_name,
@@ -288,7 +306,7 @@ class KnativeServingBackend:
         except Exception:
             pass
 
-        self.api.create_namespaced_custom_object(
+        self.custom_api.create_namespaced_custom_object(
                     group="tekton.dev",
                     version="v1alpha1",
                     namespace=self.namespace,
@@ -299,7 +317,7 @@ class KnativeServingBackend:
         logger.debug("Building runtime...")
         pod_name = None
         w = watch.Watch()
-        for event in w.stream(self.api.list_namespaced_custom_object, namespace=self.namespace,
+        for event in w.stream(self.custom_api.list_namespaced_custom_object, namespace=self.namespace,
                               group="tekton.dev", version="v1alpha1", plural="taskruns",
                               field_selector="metadata.name={0}".format(task_run_name)):
             if event['object'].get('status'):
@@ -310,7 +328,7 @@ class KnativeServingBackend:
             raise Exception('Unable to get the pod name from the task that is building the runtime')
 
         w = watch.Watch()
-        for event in w.stream(self.v1.list_namespaced_pod, namespace=self.namespace,
+        for event in w.stream(self.core_api.list_namespaced_pod, namespace=self.namespace,
                               field_selector="metadata.name={0}".format(pod_name)):
             if event['object'].status.phase == "Succeeded":
                 w.stop()
@@ -319,14 +337,15 @@ class KnativeServingBackend:
                 logger.debug('Something went wrong building the default Lithops runtime with Tekton')
                 for container in event['object'].status.container_statuses:
                     if container.state.terminated.reason == 'Error':
-                        logs = self.v1.read_namespaced_pod_log(name=pod_name,
-                                                               container=container.name,
-                                                               namespace=self.namespace)
+                        logs = self.core_api.read_namespaced_pod_log(
+                                    name=pod_name,
+                                    container=container.name,
+                                    namespace=self.namespace)
                         logger.debug("Tekton container '{}' failed: {}".format(container.name, logs.strip()))
 
                 raise Exception('Unable to build the default Lithops runtime with Tekton')
 
-        self.api.delete_namespaced_custom_object(
+        self.custom_api.delete_namespaced_custom_object(
                     group="tekton.dev",
                     version="v1alpha1",
                     name=task_run_name,
@@ -341,25 +360,66 @@ class KnativeServingBackend:
         """
         Builds the default runtime
         """
-        if os.system('docker --version >{} 2>&1'.format(os.devnull)) == 0:
+        if os.system('{} --version >{} 2>&1'.format(kconfig.DOCKER_PATH, os.devnull)) == 0:
             # Build default runtime using local dokcer
-            python_version = version_str(sys.version_info).replace('.', '')
-            location = 'https://raw.githubusercontent.com/lithops-cloud/lithops/master/runtime/knative'
-            resp = requests.get('{}/Dockerfile.python{}'.format(location, python_version))
-            dockerfile = "Dockefile.default-kantive-runtime"
-            if resp.status_code == 200:
-                with open(dockerfile, 'w') as f:
-                    f.write(resp.text)
-                self.build_runtime(default_runtime_img_name, dockerfile)
-                os.remove(dockerfile)
-            else:
-                msg = 'There was an error fetching the default runitme Dockerfile: {}'.format(resp.text)
-                logger.error(msg)
-                exit()
+            python_version = version_str(sys.version_info)
+            dockerfile = "Dockefile.default-knative-runtime"
+            with open(dockerfile, 'w') as f:
+                f.write("FROM python:{}-slim-buster\n".format(python_version))
+                f.write(kconfig.DEFAULT_DOCKERFILE)
+            self.build_runtime(default_runtime_img_name, dockerfile)
+            os.remove(dockerfile)
         else:
             # Build default runtime using Tekton
             self._build_default_runtime_from_git(default_runtime_img_name)
 
+    def _create_container_registry_secret(self):
+        """
+        Create the container registry secret in the cluster
+        (only if credentials are present in config)
+        """
+        if not all(key in self.knative_config for key in ["docker_user", "docker_password"]):
+            return
+
+        logger.debug('Creating container registry secret')
+        docker_server = self.knative_config.get('docker_server', 'https://index.docker.io/v1/')
+        docker_user = self.knative_config.get('docker_user')
+        docker_password = self.knative_config.get('docker_password')
+
+        cred_payload = {
+            "auths": {
+                docker_server: {
+                    "Username": docker_user,
+                    "Password": docker_password
+                }
+            }
+        }
+
+        data = {
+            ".dockerconfigjson": base64.b64encode(
+                json.dumps(cred_payload).encode()
+            ).decode()
+        }
+
+        secret = client.V1Secret(
+            api_version="v1",
+            data=data,
+            kind="Secret",
+            metadata=dict(name="lithops-regcred", namespace=self.namespace),
+            type="kubernetes.io/dockerconfigjson",
+        )
+
+        try:
+            self.coreV1Api.delete_namespaced_secret("lithops-regcred", self.namespace)
+        except ApiException as e:
+            pass
+
+        try:
+            self.coreV1Api.create_namespaced_secret(self.namespace, secret)
+        except ApiException as e:
+            if e.status != 409:
+                raise e
+
     def _create_service(self, docker_image_name, runtime_memory, timeout):
         """
         Creates a service in knative based on the docker_image_name and the memory provided
@@ -375,16 +435,22 @@ class KnativeServingBackend:
         logger.debug("Namespace: {}".format(self.namespace))
 
         svc_res['spec']['template']['spec']['timeoutSeconds'] = timeout
-        full_docker_image_name = '/'.join([self.knative_config['docker_repo'], docker_image_name])
-        svc_res['spec']['template']['spec']['containers'][0]['image'] = full_docker_image_name
-        svc_res['spec']['template']['spec']['containers'][0]['resources']['limits']['memory'] = '{}Mi'.format(runtime_memory)
-        svc_res['spec']['template']['spec']['containers'][0]['resources']['limits']['cpu'] = '{}m'.format(self.knative_config['cpu'])
+        svc_res['spec']['template']['spec']['containerConcurrency'] = 1
+
+        container = svc_res['spec']['template']['spec']['containers'][0]
+        container['image'] = docker_image_name
+        container['env'][0] = {'name': 'CONCURRENCY', 'value': '1'}
+        container['env'][1] = {'name': 'TIMEOUT', 'value': str(timeout)}
+        container['resources']['limits']['memory'] = '{}Mi'.format(runtime_memory)
+        container['resources']['limits']['cpu'] = str(self.knative_config['runtime_cpu'])
+        container['resources']['requests']['memory'] = '{}Mi'.format(runtime_memory)
+        container['resources']['requests']['cpu'] = str(self.knative_config['runtime_cpu'])
 
         try:
             # delete the service resource if exists
-            self.api.delete_namespaced_custom_object(
-                    group="serving.knative.dev",
-                    version="v1alpha1",
+            self.custom_api.delete_namespaced_custom_object(
+                    group=kconfig.DEFAULT_GROUP,
+                    version=kconfig.DEFAULT_VERSION,
                     name=service_name,
                     namespace=self.namespace,
                     plural="services",
@@ -395,18 +461,18 @@ class KnativeServingBackend:
             pass
 
         # create the service resource
-        self.api.create_namespaced_custom_object(
-                group="serving.knative.dev",
-                version="v1alpha1",
+        self.custom_api.create_namespaced_custom_object(
+                group=kconfig.DEFAULT_GROUP,
+                version=kconfig.DEFAULT_VERSION,
                 namespace=self.namespace,
                 plural="services",
                 body=svc_res
             )
 
         w = watch.Watch()
-        for event in w.stream(self.api.list_namespaced_custom_object,
-                              namespace=self.namespace, group="serving.knative.dev",
-                              version="v1alpha1", plural="services",
+        for event in w.stream(self.custom_api.list_namespaced_custom_object,
+                              namespace=self.namespace, group=kconfig.DEFAULT_GROUP,
+                              version=kconfig.DEFAULT_VERSION, plural="services",
                               field_selector="metadata.name={0}".format(service_name),
                               timeout_seconds=300):
             if event['object'].get('status'):
@@ -434,10 +500,11 @@ class KnativeServingBackend:
         """
         Extract installed Python modules from docker image
         """
+        logger.info("Extracting Python modules from: {}".format(docker_image_name))
         payload = {}
 
         payload['service_route'] = "/preinstalls"
-        logger.debug("Extracting Python modules list from: {}".format(docker_image_name))
+
         try:
             runtime_meta = self.invoke(docker_image_name, memory, payload, return_result=True)
         except Exception as e:
@@ -448,7 +515,7 @@ class KnativeServingBackend:
 
         return runtime_meta
 
-    def create_runtime(self, docker_image_name, memory, timeout=kconfig.RUNTIME_TIMEOUT_DEFAULT):
+    def create_runtime(self, docker_image_name, memory, timeout):
         """
         Creates a new runtime into the knative default namespace from an already built Docker image.
         As knative does not have a default image already published in a docker registry, lithops
@@ -462,6 +529,7 @@ class KnativeServingBackend:
             docker_image_name = default_runtime_img_name
             self._build_default_runtime(default_runtime_img_name)
 
+        self._create_container_registry_secret()
         self._create_service(docker_image_name, memory, timeout)
         runtime_meta = self._generate_runtime_meta(docker_image_name, memory)
 
@@ -474,24 +542,28 @@ class KnativeServingBackend:
         """
         Builds a new runtime from a Docker file and pushes it to the Docker hub
         """
-        logger.info('Building a new docker image from Dockerfile')
-        logger.info('Docker image name: {}'.format(docker_image_name))
+        logger.debug('Building a new docker image from Dockerfile')
+        logger.debug('Docker image name: {}'.format(docker_image_name))
 
         expression = '^([a-z0-9]+)/([-a-z0-9]+)(:[a-z0-9]+)?'
         result = re.match(expression, docker_image_name)
 
         if not result or result.group() != docker_image_name:
-            raise Exception("Invalid docker image name: '.' or '_' characters are not allowed")
+            raise Exception("Invalid docker image name: All letters must be "
+                            "lowercase and '.' or '_' characters are not allowed")
 
         entry_point = os.path.join(os.path.dirname(__file__), 'entry_point.py')
-        create_function_handler_zip(kconfig.FH_ZIP_LOCATION, entry_point, 'lithopsproxy.py')
+        create_handler_zip(kconfig.FH_ZIP_LOCATION, entry_point, 'lithopsproxy.py')
 
         if dockerfile:
-            cmd = 'docker build -t {} -f {} .'.format(docker_image_name, dockerfile)
+            cmd = '{} build -t {} -f {} .'.format(kconfig.DOCKER_PATH,
+                                                  docker_image_name,
+                                                  dockerfile)
         else:
-            cmd = 'docker build -t {} .'.format(docker_image_name)
+            cmd = '{} build -t {} .'.format(kconfig.DOCKER_PATH, docker_image_name)
 
-        if not self.log_active:
+        logger.info('Building default runtime')
+        if logger.getEffectiveLevel() != logging.DEBUG:
             cmd = cmd + " >{} 2>&1".format(os.devnull)
 
         res = os.system(cmd)
@@ -500,8 +572,8 @@ class KnativeServingBackend:
 
         self._delete_function_handler_zip()
 
-        cmd = 'docker push {}'.format(docker_image_name)
-        if not self.log_active:
+        cmd = '{} push {}'.format(kconfig.DOCKER_PATH, docker_image_name)
+        if logger.getEffectiveLevel() != logging.DEBUG:
             cmd = cmd + " >{} 2>&1".format(os.devnull)
         res = os.system(cmd)
         if res != 0:
@@ -511,9 +583,9 @@ class KnativeServingBackend:
         service_name = self._format_service_name(docker_image_name, memory)
         logger.info('Deleting runtime: {}'.format(service_name))
         try:
-            self.api.delete_namespaced_custom_object(
-                    group="serving.knative.dev",
-                    version="v1alpha1",
+            self.custom_api.delete_namespaced_custom_object(
+                    group=kconfig.DEFAULT_GROUP,
+                    version=kconfig.DEFAULT_VERSION,
                     name=service_name,
                     namespace=self.namespace,
                     plural="services",
@@ -522,7 +594,7 @@ class KnativeServingBackend:
         except Exception:
             pass
 
-    def delete_all_runtimes(self):
+    def clean(self):
         """
         Deletes all runtimes deployed in knative
         """
@@ -535,9 +607,9 @@ class KnativeServingBackend:
         List all the runtimes deployed in knative
         return: list of tuples [docker_image_name, memory]
         """
-        knative_services = self.api.list_namespaced_custom_object(
-                                group="serving.knative.dev",
-                                version="v1alpha1",
+        knative_services = self.custom_api.list_namespaced_custom_object(
+                                group=kconfig.DEFAULT_GROUP,
+                                version=kconfig.DEFAULT_VERSION,
                                 namespace=self.namespace,
                                 plural="services"
                             )
@@ -574,8 +646,11 @@ class KnativeServingBackend:
         else:
             endpoint = 'http://{}'.format(service_host)
 
+        if 'codeengine' in endpoint:
+            endpoint = endpoint.replace('http://', 'https://')
+
         exec_id = payload.get('executor_id')
-        call_id = payload.get('call_id')
+        call_ids = payload.get('call_ids')
         job_id = payload.get('job_id')
         route = payload.get("service_route", '/')
 
@@ -588,18 +663,19 @@ class KnativeServingBackend:
             else:
                 conn = http.client.HTTPConnection(parsed_url.netloc)
 
-            conn.request("POST", route, body=json.dumps(payload), headers=headers)
-
-            if exec_id and job_id and call_id:
-                logger.debug('ExecutorID {} | JobID {} - Function call {} invoked'
-                             .format(exec_id, job_id, call_id))
+            if exec_id and job_id and call_ids:
+                logger.debug('ExecutorID {} | JobID {} - Invoking function call {}'
+                             .format(exec_id, job_id, ', '.join(call_ids)))
             elif exec_id and job_id:
-                logger.debug('ExecutorID {} | JobID {} - Function invoked'
+                logger.debug('ExecutorID {} | JobID {} - Invoking function'
                              .format(exec_id, job_id))
             else:
-                logger.debug('Function invoked')
+                logger.debug('Invoking function')
+
+            conn.request("POST", route, body=json.dumps(payload, default=str), headers=headers)
 
             resp = conn.getresponse()
+            headers = dict(resp.getheaders())
             resp_status = resp.status
             resp_data = resp.read().decode("utf-8")
             conn.close()
@@ -615,7 +691,7 @@ class KnativeServingBackend:
             raise Exception("Lithops runtime is not deployed in your k8s cluster")
         else:
             logger.debug('ExecutorID {} | JobID {} - Function call {} failed ({}). Retrying request'
-                         .format(exec_id, job_id, call_id, resp_data.replace('.', '')))
+                         .format(exec_id, job_id, ', '.join(call_ids), resp_status))
 
     def get_runtime_key(self, docker_image_name, runtime_memory):
         """
@@ -624,6 +700,7 @@ class KnativeServingBackend:
         in order to know which runtimes are installed and which not.
         """
         service_name = self._format_service_name(docker_image_name, runtime_memory)
-        runtime_key = os.path.join(self.cluster, self.namespace, service_name)
+        cluster = self.cluster.replace('https://', '').replace('http://', '')
+        runtime_key = os.path.join(self.name, cluster, self.namespace, service_name)
 
         return runtime_key
