@@ -1,5 +1,5 @@
 #
-# Copyright Cloudlab URV 2020
+# (C) Copyright Cloudlab URV 2020
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,16 +14,19 @@
 # limitations under the License.
 #
 
+REQ_PARAMS = ('project_name', 'service_account', 'credentials_path', 'region')
+
+
 def load_config(config_data=None):
     if 'gcp' not in config_data:
-        raise Exception("'gcp' section is mandatory in the configuration")
+        raise Exception("gcp section is mandatory in the configuration")
 
-    required_parameters_0 = ('project_name', 'service_account', 'credentials_path')
-    if not set(required_parameters_0) <= set(config_data['gcp']):
-        raise Exception("'project_name', 'service_account' and 'credentials_path' "
-        "are mandatory under 'gcp' section")
-
-    if 'region' not in config_data['gcp']:
-        config_data['gcp']['region'] = config_data['lithops']['compute_backend_region']
+    for param in REQ_PARAMS:
+        if param not in config_data['gcp']:
+            msg = f"'{param}' is mandatory under 'gcp' section of the configuration"
+            raise Exception(msg)
 
     config_data['gcp_storage'] = config_data['gcp'].copy()
+
+    if 'storage_bucket' in config_data['gcp_storage']:
+        config_data['lithops']['storage_bucket'] = config_data['gcp_storage']['storage_bucket']
